import re
from typing import Iterator

from rich.console import Console
from rich.style import Style
from rich.syntax import Syntax

from gl_search.models import RepoResult


class MatchFinder:
    match_position: list[tuple[int, int]]

    def __init__(self, text: str, search_code_input: str) -> None:
        self.match_position = []
        self._process(text, search_code_input)

    def _process(self, text: str, search_code_input: str) -> None:
        splitted_lines = text.split("\n")

        accumulator = 0

        for line, splitted_line in enumerate(splitted_lines, start=1):
            accumulator += len(splitted_line)

            regex_findall_result: Iterator[re.Match[str]] = re.finditer(
                rf"{search_code_input}", splitted_line, flags=re.IGNORECASE
            )

            for match in regex_findall_result:
                self.match_position.append(((line, match.start()), (line, match.end())))


def print_results(results: list[RepoResult], search_code_input: str) -> None:
    console = Console()

    for entry in results:
        if not entry.results:
            continue

        console.print(f"Proj : {entry.name}\n")

        for content in entry.results:
            match_finder = MatchFinder(content.data, search_code_input)

            text = Syntax(
                content.data, content.path.split(".")[-1], start_line=content.start_line, line_numbers=True
            )

            for match_position in match_finder.match_position:
                text.stylize_range(Style(bgcolor="deep_pink4"), *match_position)

            console.print(f"{content.filename} - {entry.web_url}/-/blob/{content.ref}/{content.path}\n")
            console.print(text)
            console.print("-----------\n")



import time
from http import HTTPStatus
from random import uniform
from typing import Callable, Final

import requests

from .config import settings
from .models import RequestDescribe


def retrieve_data(
    request: RequestDescribe,
    transform_data: Callable = lambda value: value,
    max_random_time_for_sleep: float = 0,
) -> list:
    count: int = 0
    data_list: list[dict] = []
    HEADERS: Final[dict[str, str]] = {"PRIVATE-TOKEN": settings.GITLAB_PRIVATE_TOKEN}
    while True or count < settings.MAX_DEEP_SEARCH:
        count += 1
        response: requests.Response = requests.get(
            request.url, params=request.params, headers=HEADERS, timeout=10
        )

        if not response.status_code == HTTPStatus.OK:
            raise Exception(f"invalid status code {response.status_code}")

        data_list.extend([transform_data(data) for data in response.json()])

        try:
            request.url: str = response.links["next"]["url"]
            request.params = {}
        except KeyError:
            break

        time.sleep(uniform(0, max_random_time_for_sleep))

    return data_list



from typing import Optional

import click

from gl_search.display import print_results
from gl_search.models import SearchParams
from gl_search.search import search


@click.group()
def search_cli() -> None:
    ...


@search_cli.command(name="search")
@click.option("-p", "--path", default=None, help="search by Path")
@click.option("-fn", "--filename", default=None, help="search by filename")
@click.option("-ext", "--extension", default=None, help="code filename extension :: py,js,cs")
@click.option("-g", "--groups", default=None, help="search by gitlab group")
@click.option("-mw", "--max-workers", default=5, type=int, help="number of parallel requests")
@click.option(
    "-v",
    "--visibility",
    type=click.Choice(["internal", "public", "private"], case_sensitive=False),
    multiple=True,
    default=["internal", "public", "private"],
    help="repositories visibility",
)
@click.option("-xdr", "--max-delay-request", default=5, type=int, help="max delay request")
@click.argument("search_code_input")
def search_command(
    groups: Optional[str],
    search_code_input: str,
    max_workers: int,
    visibility: list[str],
    extension: Optional[str],
    filename: Optional[str],
    path: Optional[str],
    max_delay_request: int,
) -> None:
    """Search command."""
    results: list[dict[str, str]] = search(
        SearchParams(
            groups=groups,
            search_code_input=search_code_input,
            max_workers=max_workers,
            visibility=visibility,
            extension=extension,
            filename=filename,
            path=path,
            max_random_time_for_sleep=max_delay_request,
        )
    )

    print_results(results, search_code_input)



import os
from typing import Final

from dynaconf import Dynaconf, Validator

BLOCK_SETTINGS_NAME: Final[str] = "gl-settings"

SETTINGS_FILE_PATH: Final[str] = f"{os.path.expanduser('~')}/.gl-settings.toml"
settings = Dynaconf(
    envvar_prefix=False,
    load_dotenv=True,
    settings_files=[SETTINGS_FILE_PATH],
    environments=[BLOCK_SETTINGS_NAME],
    default_env=BLOCK_SETTINGS_NAME,
    validators=[
        Validator("GITLAB_URL", default="https://gitlab.com"),
        Validator("MAX_DEEP_SEARCH", default=1000),
    ],
)

settings.validators.register(
    Validator(
        "GITLAB_PRIVATE_TOKEN",
        must_exist=True,
        messages={
            "must_exist_true": "You must register your token.",
            "operations": "You must register your token.",
        },
    )
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gl_search', 'gl_search.clis']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'dynaconf>=3.1.11,<4.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'requests>=2.28.1,<3.0.0',
 'rich>=12.6.0,<13.0.0',
 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['gl-search = gl_search.cli:cli']}

setup_kwargs = {
    'name': 'gl-search',
    'version': '0.2.0',
    'description': 'Script to help search code in self-hosted gitlab',
    'long_description': '# gl-search\n\nThis is a lib that use gitlab api to search code.\nThis code is based in the lib [gitlab-search](https://github.com/phillipj/gitlab-search).\n\n## About\n\nIt is a lib to search code at gitlab.\nThis search can be parameterized with params groups, extension, filename, path, max-workers and visibility.\n\n## How install?\n\n```bash\npip install gl_search\n```\n\n## How it works?\n\nThe lib use gitlab token (GITLAB_PRIVATE_TOKEN) to search.\n\n## How to setup the token?\n\nGet your token at gitlab and then execute following command to save at home user at the .gl-settings.toml file.\n\n```bash\ngl-search setup-token <token>\n```\n\n## Can I change gitlab official to self hosted?\n\nYes you can. Use following command to setup the gitlab-address\n\n```bash\ngl-search setup-gitlab-address <self-hosted-gitlab-address>\n```\n\n## Where I get gitlab token?\n\nYou can get on following link [gitlab-token](https://gitlab.com/-/profile/personal_access_tokens)\n\n## Why this lib was built?\n\nI had problem with repo visibility using a mentioned lib above so I built this script to resolve my problem.\n\n## How to use\n\n```bash\ngl-search search test\n```\n\nThis options is show up below.\n\n```bash\n➜  gl_search git:(main) ✗ gl-search search --help\nUsage: gl-search search [OPTIONS] SEARCH_CODE_INPUT\n\n  Search command.\n\nOptions:\n  -p, --path TEXT                 search by Path\n  -fn, --filename TEXT            search by filename\n  -ext, --extension TEXT          code filename extension :: py,js,cs\n  -g, --groups TEXT               search by gitlab group\n  -mw, --max-workers INTEGER      number of parallel requests\n  -v, --visibility [internal|public|private]\n                                  repositories visibility\n  -xdr, --max-delay-request INTEGER\n  --help                          Show this message and exit.\n```\n\n## How was made the lib?\n\nThe lib was built using click, rich, request, ThreadPoolExecutor.\n',
    'author': 'Bernardo Gomes',
    'author_email': 'bgomesdeabreu@gmail.com',
    'maintainer': 'Bernardo Gomes',
    'maintainer_email': 'bgomesdeabreu@gmail.com',
    'url': 'https://github.com/Bernardoow/gl_search',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from typing import Optional

from pydantic import BaseModel, Field, HttpUrl


class SearchParams(BaseModel):
    groups: Optional[str]
    search_code_input: str
    max_workers: int
    visibility: list[str]
    extension: Optional[str]
    filename: Optional[str]
    path: Optional[str]
    max_random_time_for_sleep: int


class SearchRepoParams(BaseModel):
    repo_id: int
    search: str = Field(alias="search_code_input")
    max_random_time_for_sleep: int
    extension: Optional[str] = None
    filename: Optional[str] = None
    path: Optional[str] = None

    @property
    def search_with_params(self) -> str:
        search = self.search
        if self.extension:
            search += f" extension:{self.extension}"

        if self.filename:
            search += f" filename:{self.filename}"

        if self.path:
            search += f" path:{self.path}"

        return search


class RequestDescribe(BaseModel):
    url: HttpUrl
    params: dict[str, str] = dict()


class SearchEntryResult(BaseModel):
    path: str
    filename: str
    project_id: int
    data: str
    start_line: int = Field(alias="startline")
    ref: str


class Repo(BaseModel):
    id: int
    name: str
    visibility: str
    web_url: str

    class Config:
        frozen = True


class RepoResult(BaseModel):
    name: str
    web_url: str
    results: list[SearchEntryResult] = list()
